/*
 *  Copyright (c) xlightweb.org, 2008 - 2009. All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Please refer to the LGPL license at: http://www.gnu.org/copyleft/lesser.txt
 * The latest copy of this software may be found on http://www.xlightweb.org/
 */
package org.xlightweb;

import java.io.IOException;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.logging.Level;
import java.util.logging.Logger;





/**
 * 
 * body writer implementation base
 * 
 * @author grro@xlightweb.org
 */
abstract class AbstractNetworkMessageWriter implements IMessageWriter {
	
    private static final Logger LOG = Logger.getLogger(AbstractNetworkMessageWriter.class.getName());

	private AtomicBoolean isOpen = new AtomicBoolean(true);
    
	private final AbstractHttpConnection httpConnection;
	private final boolean destroyOnClose;
	
	 
	/**
	 * constructor 
	 * 
	 * @param httpConnection   the http connection
	 */
	public AbstractNetworkMessageWriter(AbstractHttpConnection httpConnection, boolean destroyOnClose) {
		this.httpConnection = httpConnection;
		this.destroyOnClose = destroyOnClose;
	}
	
	
	/**
	 * returns the http connection
	 * @return the http connection 
	 */
	final AbstractHttpConnection getConnection() {
		return httpConnection;
	}
	
	
	final String getId() {
		return httpConnection.getId();
	}
	
	public final void close() throws IOException {
		
		if (isOpen.getAndSet(false)) {
			onClose(destroyOnClose);
		}
	}
	
	abstract void onClose(boolean destroyOnClose) throws IOException;
	
	
	/**
	 * destroys the bodywriter. If a http connection is assigned,
	 * it will also be destroyed
	 */
	public final void destroy() {
		if (httpConnection == null) {
			return;
		}
		
		if (LOG.isLoggable(Level.FINE)) {
			LOG.fine("[" + getId() + "] destroying connection");
		}
		httpConnection.destroy();
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public int getPendingWriteDataSize() {
		return httpConnection.getUnderlyingTcpConnection().getPendingWriteDataSize();
	}
}
