/*
 *  Copyright (c) xlightweb.org, 2008 - 2009. All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Please refer to the LGPL license at: http://www.gnu.org/copyleft/lesser.txt
 * The latest copy of this software may be found on http://www.xlightweb.org/
 */
package org.xlightweb;

import java.io.IOException;
import java.nio.ByteBuffer;

import org.xsocket.connection.IWriteCompletionHandler;
import org.xsocket.connection.IConnection.FlushMode;




/**
 * body writer 
 *  
 * @author grro@xlightweb.org
 */
interface IMessageWriter {
	
	 
	/**
	 * writes data  
	 * 
	 * @param bodyData            the data to write 
	 * @param isContentImmutable  true, if content is immutable          
	 * @param flushMode           the flush mode
	 * @param completionHandler   the completion handler or <null>
	 * @throws IOException if an exception occurs
	 */
	void flush(ByteBuffer[] bodyData, boolean isContentImmutable, FlushMode flushMode, IWriteCompletionHandler completionHandler) throws IOException;	
	
	
	/**
	 * closes the writer 
	 * 
	 * @throws IOException  if an exception occurs
	 */
	void close() throws IOException;
	
	
	/**
	 * destroys the writer
	 */
	void destroy();
	
	
	/**
	 * will return true, if this writer is a network endpoint
	 * 
	 * @return true if this writer is a network endpoint
	 */
	boolean isNetworkEndpoint();
	
	
	/**
	 * returns the size of pending write data 
	 *   
	 * @return the size of pending write data
	 */
	int getPendingWriteDataSize();
}