/*
 *  Copyright (c) xlightweb.org, 2008 - 2009. All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Please refer to the LGPL license at: http://www.gnu.org/copyleft/lesser.txt
 * The latest copy of this software may be found on http://www.xlightweb.org/
 */
package org.xlightweb;

import java.io.IOException;
import java.nio.ByteBuffer;

import org.xsocket.connection.IWriteCompletionHandler;
import org.xsocket.connection.IConnection.FlushMode;




/**
 * local body writer 
 *  
 * @author grro@xlightweb.org
 */
final class LocalMessageWriter implements IMessageWriter {
	
	private final ITarget forwardTarget;
	
	
	/**
	 * constructor 
	 * 
	 * @param forwardTarget  target
	 */
	public LocalMessageWriter(ITarget forwardTarget) {
		this.forwardTarget = forwardTarget;
	}
	

	/**
     * {@inheritDoc}
     */
	public void flush(ByteBuffer[] bodyData, boolean isContentImmutable, FlushMode flushMode, IWriteCompletionHandler completionHandler) throws IOException {
		forwardTarget.append(bodyData, isContentImmutable, completionHandler);
	}
	

	/**
	 * {@inheritDoc}
	 */
	public void close() throws IOException {
		forwardTarget.setComplete();
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public void destroy() {
		forwardTarget.destroy();
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public boolean isNetworkEndpoint() {
		return false;
	}

	
	public int getPendingWriteDataSize() {
		return 0;
	}

	
	
	/**
	 * target
	 *  
	 * @author grro
	 */
	static interface ITarget {
		
		
		/**
		 * write data
		 * 
		 * @param bodyData           the data 
		 * @param completionHandler  the copmletion handler
		 * @throws IOException in an exception occurs 
		 */
		public void append(ByteBuffer[] bodyData, boolean isContentImmutable, IWriteCompletionHandler completionHandler) throws IOException;
		
		
		/**
		 * signal that the complete body is written 
		 * @throws IOException
		 */
		public void setComplete() throws IOException;
		
		
		/**
		 * destroys the target
		 */
		public void destroy();
	}
}