/*
 *  Copyright (c) xlightweb.org, 2008 - 2009. All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Please refer to the LGPL license at: http://www.gnu.org/copyleft/lesser.txt
 * The latest copy of this software may be found on http://www.xlightweb.org/
 */
package org.xlightweb;





import org.xsocket.Execution;
import org.xsocket.ILifeCycle;


 

/** 
 * <br/><br/><b>This is a xSocket internal class and subject to change</b> 
 *
 * @author grro@xlightweb.org
 */
public final class RequestHandlerInfo {
	
	
	private boolean isRequestHandlerSynchronizedOnSession = false;
	private boolean isRequestHandlerInvokeOnMessageReceived = false;
	private boolean isRequestHandlerMultithreaded = true;

	private boolean isLifeCycle = false;
	
	private boolean isRequestTimoutHandler = false;
	private boolean isRequestTimoutHandlerMultithreaded = true;

	
	

	@SuppressWarnings("unchecked")
	RequestHandlerInfo(Class clazz) {

		if (clazz == null) {
			return;
		}
		
		isLifeCycle = ILifeCycle.class.isAssignableFrom(clazz);
		

		if (IHttpRequestHandler.class.isAssignableFrom(clazz)) {
			isRequestHandlerMultithreaded = isOnRequestMultithreaded((Class<IHttpRequestHandler>) clazz);
			isRequestHandlerInvokeOnMessageReceived = isOnRequestInvokeOnMessageReceived((Class<IHttpRequestHandler>) clazz);
			isRequestHandlerSynchronizedOnSession = isOnSession((Class<IHttpRequestHandler>) clazz);			
		}
		
		if (IHttpRequestTimeoutHandler.class.isAssignableFrom(clazz)) {
			isRequestTimoutHandler = true;
			isRequestTimoutHandlerMultithreaded = HttpUtils.isRequestTimeoutHandlerMultithreaded((Class<IHttpRequestTimeoutHandler>) clazz);
		}
	}


	static boolean isOnRequestMultithreaded(Class<IHttpRequestHandler> serverHandlerClass) {
		boolean isMultithreaded = HttpUtils.isHandlerMultithreaded(serverHandlerClass, (IHttpRequestHandler.DEFAULT_EXECUTION_MODE == Execution.MULTITHREADED));
		return HttpUtils.isMethodMultithreaded(serverHandlerClass, "onRequest", isMultithreaded, IHttpExchange.class);
	}

	
	static boolean isOnRequestInvokeOnMessageReceived(Class<IHttpRequestHandler> handlerClass) {
		boolean invokeOnMessageReceived = HttpUtils.isInvokeOnMessageReceived(handlerClass, (IHttpRequestHandler.DEFAULT_INVOKE_ON_MODE == InvokeOn.MESSAGE_RECEIVED));
		return HttpUtils.isInvokeOnMessageReceived(handlerClass, "onRequest", invokeOnMessageReceived, IHttpExchange.class);	
	}

	
	static boolean isOnSession(Class<IHttpRequestHandler> handlerClass) {
		boolean isSynchronizedOnSession = HttpUtils.isSynchronizedOnSession(handlerClass, (IHttpRequestHandler.DEFAULT_SYNCHRONIZED_ON_MODE == SynchronizedOn.SESSION));
		return HttpUtils.isSynchronizedOnSession(handlerClass, "onRequest", isSynchronizedOnSession, IHttpExchange.class);
	}

	
	
	public boolean isLifeCycle() {
		return isLifeCycle;
	}
	
	public boolean isRequestHandlerInvokeOnMessageReceived() {
		return  isRequestHandlerInvokeOnMessageReceived;
	}

	
	public boolean isRequestHandlerSynchronizedOnSession() {
		return isRequestHandlerSynchronizedOnSession;
	}

	public boolean isRequestHandlerMultithreaded() {
		return isRequestHandlerMultithreaded;
	}
	
	public boolean isRequestTimeoutHandler() {
		return isRequestTimoutHandler;
	}
	
	public boolean isRequestTimeoutHandlerMultithreaded() {
		return isRequestTimoutHandlerMultithreaded;
	}
}
