/*
 *  Copyright (c) xlightweb.org, 2008 - 2009. All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Please refer to the LGPL license at: http://www.gnu.org/copyleft/lesser.txt
 * The latest copy of this software may be found on http://www.xlightweb.org/
 */
package org.xlightweb;




import java.io.IOException;
import java.net.SocketTimeoutException;

import org.xsocket.Execution;


 

/** 
 * <br/><br/><b>This is a xSocket internal class and subject to change</b> 
 *
 * @author grro@xlightweb.org
 */
public final class ResponseHandlerInfo {
	
	private boolean isResponseHandler = false;
	private boolean isResponseHandlerInvokeOnMessageReceived = false;
	private boolean isResponseHandlerMultithreaded = true;
	private boolean isResponseExeptionHandlerMultithreaded = true;
	
	private boolean isSocketTimeoutHandler = false;
	private boolean isSocketTimeoutHandlerMultithreaded = true;

	
	@SuppressWarnings("unchecked")
	ResponseHandlerInfo(Class clazz) {

		if (clazz == null) {
			return;
		}
		
		
		if (IHttpResponseHandler.class.isAssignableFrom(clazz)) {
			isResponseHandler = true;
			isResponseHandlerMultithreaded = isOnResponseMultithreaded((Class<IHttpResponseHandler>) clazz);
			isResponseHandlerInvokeOnMessageReceived = isOnResponseInvokeOnMessageReceived((Class<IHttpResponseHandler>) clazz);
			
			isResponseExeptionHandlerMultithreaded = isOnResponseExceptionMultithreaded((Class<IHttpResponseHandler>) clazz);
		}

		if (IHttpSocketTimeoutHandler.class.isAssignableFrom(clazz)) {
			isSocketTimeoutHandler = true;
			isSocketTimeoutHandlerMultithreaded = isOnResponseTimeoutMultithreaded((Class<IHttpResponseHandler>) clazz);
		}
	}


	
	static boolean isOnResponseMultithreaded(Class<IHttpResponseHandler> handlerClass) {
		boolean isMultithreaded = HttpUtils.isHandlerMultithreaded(handlerClass, (IHttpResponseHandler.DEFAULT_EXECUTION_MODE == Execution.MULTITHREADED));
		return HttpUtils.isMethodMultithreaded(handlerClass, "onResponse", isMultithreaded, IHttpResponse.class);
	}

	
	static boolean isOnResponseExceptionMultithreaded(Class<IHttpResponseHandler> handlerClass) {
		boolean isMultithreaded = HttpUtils.isHandlerMultithreaded(handlerClass, (IHttpResponseHandler.DEFAULT_EXECUTION_MODE == Execution.MULTITHREADED));
		return HttpUtils.isMethodMultithreaded(handlerClass, "onException", isMultithreaded, IOException.class);
	}
	

	static boolean isOnResponseTimeoutMultithreaded(Class<IHttpResponseHandler> handlerClass) {
		boolean isMultithreaded = HttpUtils.isHandlerMultithreaded(handlerClass, (IHttpResponseHandler.DEFAULT_EXECUTION_MODE == Execution.MULTITHREADED));
		return HttpUtils.isMethodMultithreaded(handlerClass, "onException", isMultithreaded, SocketTimeoutException.class);
	}


	static boolean isOnResponseInvokeOnMessageReceived(Class<IHttpResponseHandler> handlerClass) {
		boolean invokeOnMessageReceived = HttpUtils.isInvokeOnMessageReceived(handlerClass, (IHttpResponseHandler.DEFAULT_INVOKE_ON_MODE == InvokeOn.MESSAGE_RECEIVED));
		return HttpUtils.isInvokeOnMessageReceived(handlerClass, "onResponse", invokeOnMessageReceived, IHttpResponse.class);	
	}


	
	public boolean isResponseHandler() {
		return isResponseHandler;
	}
	
	public boolean isResponseHandlerInvokeOnMessageReceived() {
		return  isResponseHandlerInvokeOnMessageReceived;
	}

	public boolean isResponseHandlerMultithreaded() {
		return isResponseHandlerMultithreaded;
	}
	
	public boolean isSocketTimeoutHandler() {
		return isSocketTimeoutHandler;
	}

	public boolean isResponseExeptionHandlerMultithreaded() {
		return isResponseExeptionHandlerMultithreaded;
	}
	

	public boolean isSocketTimeoutHandlerMultithreaded() {
		return isSocketTimeoutHandlerMultithreaded;
	}
}
